/////////////////////////////////////////////////
// (c) 1997 Jake Birkett ... uses allegro library
/////////////////////////////////////////////////
//
//Tests Processor and Video Card speed in resolutions :
//320x200 640x480 800x600 1024x768 1280x1024 1600x1400
//
/////////////////////////////////////////////////
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "allegro.h"

#define MAX_RES         13
#define MEMLOOP		10000 // num of 320x200 memory copies to do
#define LOOP            2000


BITMAP *bmp=NULL;
BITMAP *dest=NULL;

int lastkey; // last key read from keyboard buffer

char itoa_str[33]; // used to store output string of itoa function
char buf[64];      // used to store output of sprintf function

int resx[MAX_RES]={0,320,320,320,360,360,360,640,640,800,1024,1280,1600};
int resy[MAX_RES]={0,200,240,400,360,400,480,400,480,600,768,1024,1200};
int res = 0; // resolution being tested

int cx=0; // cursor x in pixels
int cy=0; // cursor y in pixels

int i=0; // generic counter variable

int frames=0; // number of frames elapsed
long last_ticks=0; // used to calc ticks per frame
long ticks=0; // number of clock ticks (18.2/sec) since midnight

//result arrays
int success[MAX_RES];
long duration[MAX_RES];
float seconds[MAX_RES]; // ticks/18.2
int size[MAX_RES]; // size in bytes of copied memory
float sizemb[MAX_RES]; // size in megabytes of copied memory
float mb_per_sec[MAX_RES]; // the speed !!!

RGB col_black		= {0, 0, 0};
RGB col_white		= {63, 63, 63};
RGB col_red		= {63, 0, 0};
RGB col_green		= {0, 63, 0};
RGB col_blue		= {0, 0, 63};
int show_raster = 0;
/////////////////////////////
// FUNCTIONS
/////////////////////////////

/////////////////////////////
// MAIN
/////////////////////////////
void main()
{ int done = 0; // used to keep main loop going

  //SETUP
  allegro_init();
  install_keyboard();

  bmp = create_bitmap(320,200-32);
  clear(bmp);

  //MEMORY COPY
  set_gfx_mode(GFX_AUTODETECT,320,200,0,0);
  set_pallete(desktop_pallete);
  set_color(0, &col_black);
  textout_centre(screen,font,"MEMORY COPY TEST",160,0,8);
  dest = create_bitmap(320,200-32);
  last_ticks = biostime(0,0);

  //MEMORY TEST LOOP
  for (i=0;i<MEMLOOP;i++)
    blit(bmp,dest,0,0,0,32,320,200-32);

  //calc results ... use array index 0
  ticks = biostime(0,0);
  duration[0] = ticks-last_ticks;
  seconds[0] = duration[0]/18.2;
  size[0] = 320*(200-32);
  size[0] = size[0]*(MEMLOOP);
  sizemb[0] = size[0]/1024.0/1024.0;
  mb_per_sec[0]=sizemb[0]/seconds[0];
  success[res]=1;

  //RES LOOP
  for (res=1; res<MAX_RES; res++){
    if (set_gfx_mode(GFX_AUTODETECT, resx[res], resy[res], 0, 0)!=0)
      success[res]=0;
    else
    {
    success[res]=1;
    set_pallete(desktop_pallete);
    set_color(0, &col_black);

    //output screen res
    itoa(resx[res], itoa_str, 10);
    textout(screen, font, "Screen Res:", 0, 0, 8);
    textout(screen, font, itoa_str, 96, 0, 6);
    cx = 96+text_length(font, itoa_str);
    itoa(resy[res], itoa_str, 10);
    textout(screen, font, "x", cx, 0, 8);
    textout(screen, font, itoa_str, cx+8, 0, 6);

    last_ticks = biostime(0,0);

    //TEST LOOP
    for (i=0;i<LOOP;i++)
      blit(bmp,screen,0,0,0,32,320,200-32);

    //calc results
    ticks = biostime(0,0);
    duration[res] = ticks-last_ticks;
    seconds[res] = duration[res]/18.2;
    size[res] = 320*(200-32);
    size[res] = size[res]*LOOP;
    sizemb[res] = size[res]/1024.0/1024.0;
    mb_per_sec[res]=sizemb[res]/seconds[res];
    }
  } // end RES loop

  //RESULTS
  set_gfx_mode(GFX_AUTODETECT,640,480,0,0);
  set_pallete(desktop_pallete);
  set_color(0, &col_black);
  textout_centre(screen,font,"RESULTS",320,0,8);
  textout(screen,font,"Resolution",0,16,3);
  textout(screen,font,"Ticks",15*8,16,3);
  textout(screen,font,"Seconds",25*8,16,3);
  textout(screen,font,"Mb Copied",40*8,16,3);
  textout(screen,font,"Mb per Sec",55*8,16,3);
  for (res=0;res<MAX_RES;res++){
    cy = 32+res*16;
    if (res==0)
       //output memory copy test
       textout(screen,font,"Memory Copy",0,cy,6);
    else {
      //output screen res test
      itoa(resx[res],itoa_str,10);
      textout(screen,font,itoa_str,0,cy,6);
      cx = text_length(font,itoa_str);
      itoa(resy[res],itoa_str,10);
      textout(screen,font,"x",cx,cy,8);
      textout(screen,font,itoa_str,cx+8,cy,6);
    }

    //show data
    if (success[res]){
      itoa(duration[res],itoa_str,10);
      textout(screen,font,itoa_str,15*8,cy,6);
      sprintf(buf,"%f",seconds[res]);
      textout(screen,font,buf,25*8,cy,6);
      sprintf(buf,"%f",sizemb[res]);
      textout(screen,font,buf,40*8,cy,6);
      sprintf(buf,"%f",mb_per_sec[res]);
      textout(screen,font,buf,55*8,cy,6);
    }
    else
      textout(screen,font,"Failed",15*8,cy,6);
  }
  //MAIN LOOP
  while (!done){
    if (key[KEY_ESC]) done=1;
  } // end MAIN LOOP

  // EXIT
  destroy_bitmap(bmp);
  destroy_bitmap(dest);
  exit(0);
}

